// minimal reproducible example for APT hash validation bug
//
// prepare build environment:
//
//   apt-src install apt
//   apt-src build apt
//   cd apt-*
//
// place hashtest.cc in that directory and compile it with:
//
//   g++ -I obj-x86_64-linux-gnu/include/ apt-pkg/contrib/*.cc apt-pkg/aptconfiguration.cc apt-pkg/pkgsystem.cc -lsystemd -llzma -llz4 -lgcrypt -lz -lbz2 -lzstd -lresolv hashtest.cc -o hashtest
//
// run it against some file like `./hashtest file_name.bin`

#include <config.h>
#include <apt-pkg/acquire-item.h>
#include <apt-pkg/acquire-worker.h>
#include <apt-pkg/acquire.h>
#include <apt-pkg/configuration.h>
#include <apt-pkg/error.h>
#include <apt-pkg/fileutl.h>
#include <apt-pkg/hashes.h>
#include <apt-pkg/proxy.h>
#include <apt-pkg/strutl.h>
#include <algorithm>
#include <iostream>
#include <string>
#include <vector>
#include <sstream>
#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>
#include <apti18n.h>

void print_hash(std::string type, std::string filename) {
  std::clog << " - " << type << ": ";
  system((type + " " + filename + " | cut -d ' ' -f 1").c_str());
}

int main(int argc, char **argv) {
  if (argc < 2) {
    std::cout << "Give me a filename as an argument or give me death!" << std::endl;
    return 1;
  }

  std::string filename = argv[1];

  HashStringList LocalHashes;
  HashStringList ServerHashes;

  std::clog << "Calculating hashes same way apt does.\n" << std::endl;

  FileFd fd(filename, FileFd::ReadOnly, FileFd::Auto);
  Hashes LocalHashesCalc(ServerHashes);
  LocalHashesCalc.AddFD(fd);
  LocalHashes = LocalHashesCalc.GetHashStringList();

  for (HashStringList::const_iterator hs = LocalHashes.begin(); hs != LocalHashes.end(); ++hs)
    std::clog <<  " - " << hs->toStr() << std::endl;

  std::clog << "\nCalculating hashes through command line tools.\n" << std::endl;

  print_hash("md5sum", filename);
  print_hash("sha1sum", filename);
  print_hash("sha256sum", filename);
  print_hash("sha512sum", filename);

  return 0;
}
